@@ -1,8 +1,10 @@
 from .gp import *
 from .krige import *
 from .mesh import *
+from .covfunc import Observation
 
 __all__ = []
 __all__.extend(gp.__all__)
 __all__.extend(krige.__all__)
 __all__.extend(mesh.__all__)
+__all__.extend('Observation')
